diff --git a/config/checkstyle/checkstyle.xml b/config/checkstyle/checkstyle.xml
index f43a6ad0e..580780514 100644
--- a/config/checkstyle/checkstyle.xml
+++ b/config/checkstyle/checkstyle.xml
@@ -156,9 +156,13 @@
+
+
+
+
-
+
diff --git a/config/pre-commit/config.yml b/config/pre-commit/config.yml
index 9eee19122..639668531 100644
--- a/config/pre-commit/config.yml
+++ b/config/pre-commit/config.yml
@@ -2,7 +2,7 @@
# See https://pre-commit.com/hooks.html for more hooks
repos:
- repo: https://github.com/pre-commit/pre-commit-hooks
- rev: v3.2.0
+ rev: v4.0.1
hooks:
- id: trailing-whitespace
- id: end-of-file-fixer
@@ -16,7 +16,7 @@ repos:
exclude: "tsconfig\\.json$"
- repo: https://github.com/editorconfig-checker/editorconfig-checker.python
- rev: 2.3.5
+ rev: 2.3.54
hooks:
- id: editorconfig-checker
args: ['-disable-indentation']
diff --git a/package-lock.json b/package-lock.json
index 8a1a10d03..9a4786579 100644
--- a/package-lock.json
+++ b/package-lock.json
@@ -21,9 +21,9 @@
}
},
"node_modules/@rollup/plugin-typescript": {
- "version": "8.2.5",
- "resolved": "https://registry.npmjs.org/@rollup/plugin-typescript/-/plugin-typescript-8.2.5.tgz",
- "integrity": "sha512-QL/LvDol/PAGB2O0S7/+q2HpSUNodpw7z6nGn9BfoVCPOZ0r4EALrojFU29Bkoi2Hr2jgTocTejJ5GGWZfOxbQ==",
+ "version": "8.3.0",
+ "resolved": "https://registry.npmjs.org/@rollup/plugin-typescript/-/plugin-typescript-8.3.0.tgz",
+ "integrity": "sha512-I5FpSvLbtAdwJ+naznv+B4sjXZUcIvLLceYpITAn7wAP8W0wqc5noLdGIp9HGVntNhRWXctwPYrSSFQxtl0FPA==",
"dev": true,
"dependencies": {
"@rollup/pluginutils": "^3.1.0",
@@ -112,9 +112,9 @@
}
},
"node_modules/is-core-module": {
- "version": "2.6.0",
- "resolved": "https://registry.npmjs.org/is-core-module/-/is-core-module-2.6.0.tgz",
- "integrity": "sha512-wShG8vs60jKfPWpF2KZRaAtvt3a20OAn7+IJ6hLPECpSABLcKtFKTTI4ZtH5QcBruBHlq+WsdHWyz0BCZW7svQ==",
+ "version": "2.8.0",
+ "resolved": "https://registry.npmjs.org/is-core-module/-/is-core-module-2.8.0.tgz",
+ "integrity": "sha512-vd15qHsaqrRL7dtH6QNuy0ndJmRDrS9HAM1CAiSifNUFv4x1a0CCVsj18hJ1mShxIG6T2i1sO78MkP56r0nYRw==",
"dev": true,
"dependencies": {
"has": "^1.0.3"
@@ -142,9 +142,9 @@
}
},
"node_modules/preact": {
- "version": "10.5.14",
- "resolved": "https://registry.npmjs.org/preact/-/preact-10.5.14.tgz",
- "integrity": "sha512-KojoltCrshZ099ksUZ2OQKfbH66uquFoxHSbnwKbTJHeQNvx42EmC7wQVWNuDt6vC5s3nudRHFtKbpY4ijKlaQ==",
+ "version": "10.5.15",
+ "resolved": "https://registry.npmjs.org/preact/-/preact-10.5.15.tgz",
+ "integrity": "sha512-5chK29n6QcJc3m1lVrKQSQ+V7K1Gb8HeQY6FViQ5AxCAEGu3DaHffWNDkC9+miZgsLvbvU9rxbV1qinGHMHzqA==",
"funding": {
"type": "opencollective",
"url": "https://opencollective.com/preact"
@@ -177,9 +177,9 @@
}
},
"node_modules/rollup": {
- "version": "2.56.2",
- "resolved": "https://registry.npmjs.org/rollup/-/rollup-2.56.2.tgz",
- "integrity": "sha512-s8H00ZsRi29M2/lGdm1u8DJpJ9ML8SUOpVVBd33XNeEeL3NVaTiUcSBHzBdF3eAyR0l7VSpsuoVUGrRHq7aPwQ==",
+ "version": "2.60.0",
+ "resolved": "https://registry.npmjs.org/rollup/-/rollup-2.60.0.tgz",
+ "integrity": "sha512-cHdv9GWd58v58rdseC8e8XIaPUo8a9cgZpnCMMDGZFDZKEODOiPPEQFXLriWr/TjXzhPPmG5bkAztPsOARIcGQ==",
"dev": true,
"bin": {
"rollup": "dist/bin/rollup"
@@ -201,9 +201,9 @@
}
},
"node_modules/source-map-support": {
- "version": "0.5.19",
- "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.19.tgz",
- "integrity": "sha512-Wonm7zOCIJzBGQdB+thsPar0kYuCIzYvxZwlBa87yi/Mdjv7Tip2cyVbLj5o0cFPN4EVkuTwb3GDDyUx2DGnGw==",
+ "version": "0.5.21",
+ "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.21.tgz",
+ "integrity": "sha512-uBHU3L3czsIyYXKX88fdrGovxdSCoTGDRZ6SYXtSRxLZUzHg5P/66Ht6uoUlHu9EZod+inXhKo3qQgwXUT/y1w==",
"dev": true,
"dependencies": {
"buffer-from": "^1.0.0",
@@ -220,20 +220,28 @@
}
},
"node_modules/terser": {
- "version": "5.7.1",
- "resolved": "https://registry.npmjs.org/terser/-/terser-5.7.1.tgz",
- "integrity": "sha512-b3e+d5JbHAe/JSjwsC3Zn55wsBIM7AsHLjKxT31kGCldgbpFePaFo+PiddtO6uwRZWRw7sPXmAN8dTW61xmnSg==",
+ "version": "5.10.0",
+ "resolved": "https://registry.npmjs.org/terser/-/terser-5.10.0.tgz",
+ "integrity": "sha512-AMmF99DMfEDiRJfxfY5jj5wNH/bYO09cniSqhfoyxc8sFoYIgkJy86G04UoZU5VjlpnplVu0K6Tx6E9b5+DlHA==",
"dev": true,
"dependencies": {
"commander": "^2.20.0",
"source-map": "~0.7.2",
- "source-map-support": "~0.5.19"
+ "source-map-support": "~0.5.20"
},
"bin": {
"terser": "bin/terser"
},
"engines": {
"node": ">=10"
+ },
+ "peerDependencies": {
+ "acorn": "^8.5.0"
+ },
+ "peerDependenciesMeta": {
+ "acorn": {
+ "optional": true
+ }
}
},
"node_modules/tslib": {
@@ -242,9 +250,9 @@
"integrity": "sha512-77EbyPPpMz+FRFRuAFlWMtmgUWGe9UOG2Z25NqCwiIjRhOf5iKGuzSe5P2w1laq+FkRy4p+PCuVkJSGkzTEKVw=="
},
"node_modules/typescript": {
- "version": "4.3.5",
- "resolved": "https://registry.npmjs.org/typescript/-/typescript-4.3.5.tgz",
- "integrity": "sha512-DqQgihaQ9cUrskJo9kIyW/+g0Vxsk8cDtZ52a3NGh0YNTfpUSArXSohyUGnvbPazEPLu398C0UxmKSOrPumUzA==",
+ "version": "4.5.2",
+ "resolved": "https://registry.npmjs.org/typescript/-/typescript-4.5.2.tgz",
+ "integrity": "sha512-5BlMof9H1yGt0P8/WF+wPNw6GfctgGjXp5hkblpyT+8rkASSmkUKMXrxR0Xg8ThVCi/JnHQiKXeBaEwCeQwMFw==",
"dev": true,
"bin": {
"tsc": "bin/tsc",
@@ -257,9 +265,9 @@
},
"dependencies": {
"@rollup/plugin-typescript": {
- "version": "8.2.5",
- "resolved": "https://registry.npmjs.org/@rollup/plugin-typescript/-/plugin-typescript-8.2.5.tgz",
- "integrity": "sha512-QL/LvDol/PAGB2O0S7/+q2HpSUNodpw7z6nGn9BfoVCPOZ0r4EALrojFU29Bkoi2Hr2jgTocTejJ5GGWZfOxbQ==",
+ "version": "8.3.0",
+ "resolved": "https://registry.npmjs.org/@rollup/plugin-typescript/-/plugin-typescript-8.3.0.tgz",
+ "integrity": "sha512-I5FpSvLbtAdwJ+naznv+B4sjXZUcIvLLceYpITAn7wAP8W0wqc5noLdGIp9HGVntNhRWXctwPYrSSFQxtl0FPA==",
"dev": true,
"requires": {
"@rollup/pluginutils": "^3.1.0",
@@ -324,9 +332,9 @@
}
},
"is-core-module": {
- "version": "2.6.0",
- "resolved": "https://registry.npmjs.org/is-core-module/-/is-core-module-2.6.0.tgz",
- "integrity": "sha512-wShG8vs60jKfPWpF2KZRaAtvt3a20OAn7+IJ6hLPECpSABLcKtFKTTI4ZtH5QcBruBHlq+WsdHWyz0BCZW7svQ==",
+ "version": "2.8.0",
+ "resolved": "https://registry.npmjs.org/is-core-module/-/is-core-module-2.8.0.tgz",
+ "integrity": "sha512-vd15qHsaqrRL7dtH6QNuy0ndJmRDrS9HAM1CAiSifNUFv4x1a0CCVsj18hJ1mShxIG6T2i1sO78MkP56r0nYRw==",
"dev": true,
"requires": {
"has": "^1.0.3"
@@ -345,9 +353,9 @@
"dev": true
},
"preact": {
- "version": "10.5.14",
- "resolved": "https://registry.npmjs.org/preact/-/preact-10.5.14.tgz",
- "integrity": "sha512-KojoltCrshZ099ksUZ2OQKfbH66uquFoxHSbnwKbTJHeQNvx42EmC7wQVWNuDt6vC5s3nudRHFtKbpY4ijKlaQ=="
+ "version": "10.5.15",
+ "resolved": "https://registry.npmjs.org/preact/-/preact-10.5.15.tgz",
+ "integrity": "sha512-5chK29n6QcJc3m1lVrKQSQ+V7K1Gb8HeQY6FViQ5AxCAEGu3DaHffWNDkC9+miZgsLvbvU9rxbV1qinGHMHzqA=="
},
"requirejs": {
"version": "2.3.6",
@@ -366,9 +374,9 @@
}
},
"rollup": {
- "version": "2.56.2",
- "resolved": "https://registry.npmjs.org/rollup/-/rollup-2.56.2.tgz",
- "integrity": "sha512-s8H00ZsRi29M2/lGdm1u8DJpJ9ML8SUOpVVBd33XNeEeL3NVaTiUcSBHzBdF3eAyR0l7VSpsuoVUGrRHq7aPwQ==",
+ "version": "2.60.0",
+ "resolved": "https://registry.npmjs.org/rollup/-/rollup-2.60.0.tgz",
+ "integrity": "sha512-cHdv9GWd58v58rdseC8e8XIaPUo8a9cgZpnCMMDGZFDZKEODOiPPEQFXLriWr/TjXzhPPmG5bkAztPsOARIcGQ==",
"dev": true,
"requires": {
"fsevents": "~2.3.2"
@@ -381,9 +389,9 @@
"dev": true
},
"source-map-support": {
- "version": "0.5.19",
- "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.19.tgz",
- "integrity": "sha512-Wonm7zOCIJzBGQdB+thsPar0kYuCIzYvxZwlBa87yi/Mdjv7Tip2cyVbLj5o0cFPN4EVkuTwb3GDDyUx2DGnGw==",
+ "version": "0.5.21",
+ "resolved": "https://registry.npmjs.org/source-map-support/-/source-map-support-0.5.21.tgz",
+ "integrity": "sha512-uBHU3L3czsIyYXKX88fdrGovxdSCoTGDRZ6SYXtSRxLZUzHg5P/66Ht6uoUlHu9EZod+inXhKo3qQgwXUT/y1w==",
"dev": true,
"requires": {
"buffer-from": "^1.0.0",
@@ -399,14 +407,14 @@
}
},
"terser": {
- "version": "5.7.1",
- "resolved": "https://registry.npmjs.org/terser/-/terser-5.7.1.tgz",
- "integrity": "sha512-b3e+d5JbHAe/JSjwsC3Zn55wsBIM7AsHLjKxT31kGCldgbpFePaFo+PiddtO6uwRZWRw7sPXmAN8dTW61xmnSg==",
+ "version": "5.10.0",
+ "resolved": "https://registry.npmjs.org/terser/-/terser-5.10.0.tgz",
+ "integrity": "sha512-AMmF99DMfEDiRJfxfY5jj5wNH/bYO09cniSqhfoyxc8sFoYIgkJy86G04UoZU5VjlpnplVu0K6Tx6E9b5+DlHA==",
"dev": true,
"requires": {
"commander": "^2.20.0",
"source-map": "~0.7.2",
- "source-map-support": "~0.5.19"
+ "source-map-support": "~0.5.20"
}
},
"tslib": {
@@ -415,9 +423,9 @@
"integrity": "sha512-77EbyPPpMz+FRFRuAFlWMtmgUWGe9UOG2Z25NqCwiIjRhOf5iKGuzSe5P2w1laq+FkRy4p+PCuVkJSGkzTEKVw=="
},
"typescript": {
- "version": "4.3.5",
- "resolved": "https://registry.npmjs.org/typescript/-/typescript-4.3.5.tgz",
- "integrity": "sha512-DqQgihaQ9cUrskJo9kIyW/+g0Vxsk8cDtZ52a3NGh0YNTfpUSArXSohyUGnvbPazEPLu398C0UxmKSOrPumUzA==",
+ "version": "4.5.2",
+ "resolved": "https://registry.npmjs.org/typescript/-/typescript-4.5.2.tgz",
+ "integrity": "sha512-5BlMof9H1yGt0P8/WF+wPNw6GfctgGjXp5hkblpyT+8rkASSmkUKMXrxR0Xg8ThVCi/JnHQiKXeBaEwCeQwMFw==",
"dev": true
}
}
diff --git a/src/main/java/dan200/computercraft/ComputerCraftAPIImpl.java b/src/main/java/dan200/computercraft/ComputerCraftAPIImpl.java
index c8572a4b6..eda35ee9a 100644
--- a/src/main/java/dan200/computercraft/ComputerCraftAPIImpl.java
+++ b/src/main/java/dan200/computercraft/ComputerCraftAPIImpl.java
@@ -31,6 +31,7 @@ import net.minecraft.core.BlockPos;
import net.minecraft.core.Direction;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.packs.resources.ReloadableResourceManager;
+import net.minecraft.server.packs.resources.ResourceManager;
import net.minecraft.world.level.BlockGetter;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
@@ -101,7 +102,7 @@ public final class ComputerCraftAPIImpl implements IComputerCraftAPI
@Override
public IMount createResourceMount( @Nonnull String domain, @Nonnull String subPath )
{
- ReloadableResourceManager manager = (ReloadableResourceManager) ServerLifecycleHooks.getCurrentServer().getResourceManager();
+ ResourceManager manager = ServerLifecycleHooks.getCurrentServer().getResourceManager();
ResourceMount mount = ResourceMount.get( domain, subPath, manager );
return mount.exists( "" ) ? mount : null;
}
diff --git a/src/main/java/dan200/computercraft/api/lua/ILuaContext.java b/src/main/java/dan200/computercraft/api/lua/ILuaContext.java
index 70502b535..4d29b57d1 100644
--- a/src/main/java/dan200/computercraft/api/lua/ILuaContext.java
+++ b/src/main/java/dan200/computercraft/api/lua/ILuaContext.java
@@ -40,5 +40,8 @@ public interface ILuaContext
* @throws LuaException If the task could not be queued, or if the task threw an exception.
*/
@Nonnull
- MethodResult executeMainThreadTask( @Nonnull ILuaTask task ) throws LuaException;
+ default MethodResult executeMainThreadTask( @Nonnull ILuaTask task ) throws LuaException
+ {
+ return TaskCallback.make( this, task );
+ }
}
diff --git a/src/main/java/dan200/computercraft/core/asm/TaskCallback.java b/src/main/java/dan200/computercraft/api/lua/TaskCallback.java
similarity index 58%
rename from src/main/java/dan200/computercraft/core/asm/TaskCallback.java
rename to src/main/java/dan200/computercraft/api/lua/TaskCallback.java
index 1298391e4..85870a221 100644
--- a/src/main/java/dan200/computercraft/core/asm/TaskCallback.java
+++ b/src/main/java/dan200/computercraft/api/lua/TaskCallback.java
@@ -1,16 +1,14 @@
/*
- * This file is part of ComputerCraft - http://www.computercraft.info
- * Copyright Daniel Ratcliffe, 2011-2021. Do not distribute without permission.
- * Send enquiries to dratcliffe@gmail.com
+ * This file is part of the public ComputerCraft API - http://www.computercraft.info
+ * Copyright Daniel Ratcliffe, 2011-2021. This API may be redistributed unmodified and in full only.
+ * For help using the API, and posting your mods, visit the forums at computercraft.info.
*/
-package dan200.computercraft.core.asm;
-
-import dan200.computercraft.api.lua.*;
+package dan200.computercraft.api.lua;
import javax.annotation.Nonnull;
import java.util.Arrays;
-public final class TaskCallback implements ILuaCallback
+final class TaskCallback implements ILuaCallback
{
private final MethodResult pull = MethodResult.pullEvent( "task_complete", this );
private final long task;
@@ -47,19 +45,7 @@ public final class TaskCallback implements ILuaCallback
}
}
- static Object[] checkUnwrap( MethodResult result )
- {
- if( result.getCallback() != null )
- {
- // Due to how tasks are implemented, we can't currently return a MethodResult. This is an
- // entirely artificial limitation - we can remove it if it ever becomes an issue.
- throw new IllegalStateException( "Cannot return MethodResult for mainThread task." );
- }
-
- return result.getResult();
- }
-
- public static MethodResult make( ILuaContext context, ILuaTask func ) throws LuaException
+ static MethodResult make( ILuaContext context, ILuaTask func ) throws LuaException
{
long task = context.issueMainThreadTask( func );
return new TaskCallback( task ).pull;
diff --git a/src/main/java/dan200/computercraft/api/peripheral/GenericPeripheral.java b/src/main/java/dan200/computercraft/api/peripheral/GenericPeripheral.java
new file mode 100644
index 000000000..4140597ec
--- /dev/null
+++ b/src/main/java/dan200/computercraft/api/peripheral/GenericPeripheral.java
@@ -0,0 +1,45 @@
+/*
+ * This file is part of the public ComputerCraft API - http://www.computercraft.info
+ * Copyright Daniel Ratcliffe, 2011-2021. This API may be redistributed unmodified and in full only.
+ * For help using the API, and posting your mods, visit the forums at computercraft.info.
+ */
+package dan200.computercraft.api.peripheral;
+
+import dan200.computercraft.api.lua.GenericSource;
+import net.minecraft.world.level.block.entity.BlockEntity;
+import net.minecraftforge.items.IItemHandler;
+
+import javax.annotation.Nonnull;
+
+/**
+ * A {@link GenericSource} which provides methods for a peripheral.
+ *
+ * Unlike a {@link GenericSource}, all methods should target the same type, for instance a
+ * {@link BlockEntity} subclass or a capability interface. This is not currently enforced.
+ */
+public interface GenericPeripheral extends GenericSource
+{
+ /**
+ * Get the type of the exposed peripheral.
+ *
+ * Unlike normal {@link IPeripheral}s, {@link GenericPeripheral} do not have to have a type. By default, the
+ * resulting peripheral uses the resource name of the wrapped {@link BlockEntity} (for instance {@literal minecraft:chest}).
+ *
+ * However, in some cases it may be more appropriate to specify a more readable name. Overriding this method allows
+ * you to do so.
+ *
+ * When multiple {@link GenericPeripheral}s return a non-empty peripheral type for a single tile entity, the
+ * lexicographically smallest will be chosen. In order to avoid this conflict, this method should only be
+ * implemented when your peripheral targets a single tile entity AND it's likely that you're the
+ * only mod to do so. Similarly this should NOT be implemented when your methods target a
+ * capability or other interface (i.e. {@link IItemHandler}).
+ *
+ * @return The type of this peripheral or {@link PeripheralType#untyped()}.
+ * @see IPeripheral#getType()
+ */
+ @Nonnull
+ default PeripheralType getType()
+ {
+ return PeripheralType.untyped();
+ }
+}
diff --git a/src/main/java/dan200/computercraft/api/peripheral/PeripheralType.java b/src/main/java/dan200/computercraft/api/peripheral/PeripheralType.java
new file mode 100644
index 000000000..2780534c2
--- /dev/null
+++ b/src/main/java/dan200/computercraft/api/peripheral/PeripheralType.java
@@ -0,0 +1,62 @@
+/*
+ * This file is part of the public ComputerCraft API - http://www.computercraft.info
+ * Copyright Daniel Ratcliffe, 2011-2021. This API may be redistributed unmodified and in full only.
+ * For help using the API, and posting your mods, visit the forums at computercraft.info.
+ */
+package dan200.computercraft.api.peripheral;
+
+import com.google.common.base.Strings;
+
+import javax.annotation.Nonnull;
+import javax.annotation.Nullable;
+
+/**
+ * The type of a {@link GenericPeripheral}.
+ *
+ * When determining the final type of the resulting peripheral, the union of all types is taken, with the
+ * lexicographically smallest non-empty name being chosen.
+ */
+public final class PeripheralType
+{
+ private static final PeripheralType UNTYPED = new PeripheralType( null );
+
+ private final String type;
+
+ public PeripheralType( String type )
+ {
+ this.type = type;
+ }
+
+ /**
+ * An empty peripheral type, used when a {@link GenericPeripheral} does not have an explicit type.
+ *
+ * @return The empty peripheral type.
+ */
+ public static PeripheralType untyped()
+ {
+ return UNTYPED;
+ }
+
+ /**
+ * Create a new non-empty peripheral type.
+ *
+ * @param type The name of the type.
+ * @return The constructed peripheral type.
+ */
+ public static PeripheralType ofType( @Nonnull String type )
+ {
+ if( Strings.isNullOrEmpty( type ) ) throw new IllegalArgumentException( "type cannot be null or empty" );
+ return new PeripheralType( type );
+ }
+
+ /**
+ * Get the name of this peripheral type. This may be {@literal null}.
+ *
+ * @return The type of this peripheral.
+ */
+ @Nullable
+ public String getPrimaryType()
+ {
+ return type;
+ }
+}
diff --git a/src/main/java/dan200/computercraft/client/render/DebugOverlay.java b/src/main/java/dan200/computercraft/client/render/DebugOverlay.java
new file mode 100644
index 000000000..770a037b9
--- /dev/null
+++ b/src/main/java/dan200/computercraft/client/render/DebugOverlay.java
@@ -0,0 +1,58 @@
+/*
+ * This file is part of ComputerCraft - http://www.computercraft.info
+ * Copyright Daniel Ratcliffe, 2011-2021. Do not distribute without permission.
+ * Send enquiries to dratcliffe@gmail.com
+ */
+package dan200.computercraft.client.render;
+
+import dan200.computercraft.ComputerCraft;
+import dan200.computercraft.api.turtle.ITurtleUpgrade;
+import dan200.computercraft.api.turtle.TurtleSide;
+import dan200.computercraft.shared.peripheral.monitor.TileMonitor;
+import dan200.computercraft.shared.turtle.blocks.TileTurtle;
+import net.minecraft.client.Minecraft;
+import net.minecraft.world.level.block.entity.BlockEntity;
+import net.minecraft.world.phys.BlockHitResult;
+import net.minecraft.world.phys.HitResult;
+import net.minecraftforge.api.distmarker.Dist;
+import net.minecraftforge.client.event.RenderGameOverlayEvent;
+import net.minecraftforge.eventbus.api.SubscribeEvent;
+import net.minecraftforge.fml.common.Mod;
+
+import java.util.List;
+
+@Mod.EventBusSubscriber( modid = ComputerCraft.MOD_ID, value = Dist.CLIENT )
+public class DebugOverlay
+{
+ @SubscribeEvent
+ public static void onRenderText( RenderGameOverlayEvent.Text event )
+ {
+ Minecraft minecraft = Minecraft.getInstance();
+ if( !minecraft.options.renderDebug || minecraft.level == null ) return;
+ if( minecraft.hitResult == null || minecraft.hitResult.getType() != HitResult.Type.BLOCK ) return;
+
+ BlockEntity tile = minecraft.level.getBlockEntity( ((BlockHitResult) minecraft.hitResult).getBlockPos() );
+
+ if( tile instanceof TileMonitor monitor )
+ {
+ event.getRight().add( "" );
+ event.getRight().add(
+ String.format( "Targeted monitor: (%d, %d), %d x %d", monitor.getXIndex(), monitor.getYIndex(), monitor.getWidth(), monitor.getHeight() )
+ );
+ }
+ else if( tile instanceof TileTurtle turtle )
+ {
+ event.getRight().add( "" );
+ event.getRight().add( "Targeted turtle:" );
+ event.getRight().add( String.format( "Id: %d", turtle.getComputerID() ) );
+ addTurtleUpgrade( event.getRight(), turtle, TurtleSide.LEFT );
+ addTurtleUpgrade( event.getRight(), turtle, TurtleSide.RIGHT );
+ }
+ }
+
+ private static void addTurtleUpgrade( List out, TileTurtle turtle, TurtleSide side )
+ {
+ ITurtleUpgrade upgrade = turtle.getUpgrade( side );
+ if( upgrade != null ) out.add( String.format( "Upgrade[%s]: %s", side, upgrade.getUpgradeID() ) );
+ }
+}
diff --git a/src/main/java/dan200/computercraft/core/asm/Generator.java b/src/main/java/dan200/computercraft/core/asm/Generator.java
index e68732f95..4cd2d66cf 100644
--- a/src/main/java/dan200/computercraft/core/asm/Generator.java
+++ b/src/main/java/dan200/computercraft/core/asm/Generator.java
@@ -15,6 +15,7 @@ import dan200.computercraft.api.lua.IArguments;
import dan200.computercraft.api.lua.LuaException;
import dan200.computercraft.api.lua.LuaFunction;
import dan200.computercraft.api.lua.MethodResult;
+import dan200.computercraft.api.peripheral.PeripheralType;
import org.objectweb.asm.ClassWriter;
import org.objectweb.asm.MethodVisitor;
import org.objectweb.asm.Type;
@@ -108,7 +109,7 @@ public final class Generator
if( instance == null ) continue;
if( methods == null ) methods = new ArrayList<>();
- addMethod( methods, method, annotation, instance );
+ addMethod( methods, method, annotation, null, instance );
}
for( GenericMethod method : GenericMethod.all() )
@@ -119,7 +120,7 @@ public final class Generator
if( instance == null ) continue;
if( methods == null ) methods = new ArrayList<>();
- addMethod( methods, method.method, method.annotation, instance );
+ addMethod( methods, method.method, method.annotation, method.peripheralType, instance );
}
if( methods == null ) return Collections.emptyList();
@@ -127,7 +128,7 @@ public final class Generator
return Collections.unmodifiableList( methods );
}
- private void addMethod( List> methods, Method method, LuaFunction annotation, T instance )
+ private void addMethod( List> methods, Method method, LuaFunction annotation, PeripheralType genericType, T instance )
{
if( annotation.mainThread() ) instance = wrap.apply( instance );
@@ -135,13 +136,13 @@ public final class Generator
boolean isSimple = method.getReturnType() != MethodResult.class && !annotation.mainThread();
if( names.length == 0 )
{
- methods.add( new NamedMethod<>( method.getName(), instance, isSimple ) );
+ methods.add( new NamedMethod<>( method.getName(), instance, isSimple, genericType ) );
}
else
{
for( String name : names )
{
- methods.add( new NamedMethod<>( name, instance, isSimple ) );
+ methods.add( new NamedMethod<>( name, instance, isSimple, genericType ) );
}
}
}
diff --git a/src/main/java/dan200/computercraft/core/asm/GenericMethod.java b/src/main/java/dan200/computercraft/core/asm/GenericMethod.java
index e0c916c2e..691d9b742 100644
--- a/src/main/java/dan200/computercraft/core/asm/GenericMethod.java
+++ b/src/main/java/dan200/computercraft/core/asm/GenericMethod.java
@@ -8,6 +8,8 @@ package dan200.computercraft.core.asm;
import dan200.computercraft.ComputerCraft;
import dan200.computercraft.api.lua.GenericSource;
import dan200.computercraft.api.lua.LuaFunction;
+import dan200.computercraft.api.peripheral.GenericPeripheral;
+import dan200.computercraft.api.peripheral.PeripheralType;
import javax.annotation.Nonnull;
import java.lang.reflect.Method;
@@ -18,6 +20,7 @@ import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;
+import java.util.stream.Stream;
/**
* A generic method is a method belonging to a {@link GenericSource} with a known target.
@@ -27,15 +30,17 @@ public class GenericMethod
final Method method;
final LuaFunction annotation;
final Class> target;
+ final PeripheralType peripheralType;
private static final List sources = new ArrayList<>();
private static List cache;
- GenericMethod( Method method, LuaFunction annotation, Class> target )
+ GenericMethod( Method method, LuaFunction annotation, Class> target, PeripheralType peripheralType )
{
this.method = method;
this.annotation = annotation;
this.target = target;
+ this.peripheralType = peripheralType;
}
/**
@@ -46,10 +51,28 @@ public class GenericMethod
static List all()
{
if( cache != null ) return cache;
- return cache = sources.stream()
- .flatMap( x -> Arrays.stream( x.getClass().getDeclaredMethods() ) )
- .map( method ->
- {
+ return cache = sources.stream().flatMap( GenericMethod::getMethods ).collect( Collectors.toList() );
+ }
+
+ public static synchronized void register( @Nonnull GenericSource source )
+ {
+ Objects.requireNonNull( source, "Source cannot be null" );
+
+ if( cache != null )
+ {
+ ComputerCraft.log.warn( "Registering a generic source {} after cache has been built. This source will be ignored.", cache );
+ }
+
+ sources.add( source );
+ }
+
+ private static Stream getMethods( GenericSource source )
+ {
+ Class> klass = source.getClass();
+ PeripheralType type = source instanceof GenericPeripheral ? ((GenericPeripheral) source).getType() : null;
+
+ return Arrays.stream( klass.getDeclaredMethods() )
+ .map( method -> {
LuaFunction annotation = method.getAnnotation( LuaFunction.class );
if( annotation == null ) return null;
@@ -69,22 +92,8 @@ public class GenericMethod
Class> target = Reflect.getRawType( method, types[0], false );
if( target == null ) return null;
- return new GenericMethod( method, annotation, target );
+ return new GenericMethod( method, annotation, target, type );
} )
- .filter( Objects::nonNull )
- .collect( Collectors.toList() );
- }
-
-
- public static synchronized void register( @Nonnull GenericSource source )
- {
- Objects.requireNonNull( source, "Source cannot be null" );
-
- if( cache != null )
- {
- ComputerCraft.log.warn( "Registering a generic source {} after cache has been built. This source will be ignored.", cache );
- }
-
- sources.add( source );
+ .filter( Objects::nonNull );
}
}
diff --git a/src/main/java/dan200/computercraft/core/asm/LuaMethod.java b/src/main/java/dan200/computercraft/core/asm/LuaMethod.java
index b0562835c..281ab9c56 100644
--- a/src/main/java/dan200/computercraft/core/asm/LuaMethod.java
+++ b/src/main/java/dan200/computercraft/core/asm/LuaMethod.java
@@ -13,7 +13,7 @@ import java.util.Collections;
public interface LuaMethod
{
Generator GENERATOR = new Generator<>( LuaMethod.class, Collections.singletonList( ILuaContext.class ),
- m -> ( target, context, args ) -> TaskCallback.make( context, () -> TaskCallback.checkUnwrap( m.apply( target, context, args ) ) )
+ m -> ( target, context, args ) -> context.executeMainThreadTask( () -> ResultHelpers.checkNormalResult( m.apply( target, context, args ) ) )
);
IntCache DYNAMIC = new IntCache<>(
diff --git a/src/main/java/dan200/computercraft/core/asm/NamedMethod.java b/src/main/java/dan200/computercraft/core/asm/NamedMethod.java
index ea72bb7a4..35d9c0a77 100644
--- a/src/main/java/dan200/computercraft/core/asm/NamedMethod.java
+++ b/src/main/java/dan200/computercraft/core/asm/NamedMethod.java
@@ -5,7 +5,10 @@
*/
package dan200.computercraft.core.asm;
+import dan200.computercraft.api.peripheral.PeripheralType;
+
import javax.annotation.Nonnull;
+import javax.annotation.Nullable;
public final class NamedMethod
{
@@ -13,11 +16,14 @@ public final class NamedMethod
private final T method;
private final boolean nonYielding;
- NamedMethod( String name, T method, boolean nonYielding )
+ private final PeripheralType genericType;
+
+ NamedMethod( String name, T method, boolean nonYielding, PeripheralType genericType )
{
this.name = name;
this.method = method;
this.nonYielding = nonYielding;
+ this.genericType = genericType;
}
@Nonnull
@@ -36,4 +42,10 @@ public final class NamedMethod
{
return nonYielding;
}
+
+ @Nullable
+ public PeripheralType getGenericType()
+ {
+ return genericType;
+ }
}
diff --git a/src/main/java/dan200/computercraft/core/asm/PeripheralMethod.java b/src/main/java/dan200/computercraft/core/asm/PeripheralMethod.java
index 38618442f..146896b04 100644
--- a/src/main/java/dan200/computercraft/core/asm/PeripheralMethod.java
+++ b/src/main/java/dan200/computercraft/core/asm/PeripheralMethod.java
@@ -18,7 +18,7 @@ import java.util.Arrays;
public interface PeripheralMethod
{
Generator GENERATOR = new Generator<>( PeripheralMethod.class, Arrays.asList( ILuaContext.class, IComputerAccess.class ),
- m -> ( target, context, computer, args ) -> TaskCallback.make( context, () -> TaskCallback.checkUnwrap( m.apply( target, context, computer, args ) ) )
+ m -> ( target, context, computer, args ) -> context.executeMainThreadTask( () -> ResultHelpers.checkNormalResult( m.apply( target, context, computer, args ) ) )
);
IntCache DYNAMIC = new IntCache<>(
diff --git a/src/main/java/dan200/computercraft/core/asm/ResultHelpers.java b/src/main/java/dan200/computercraft/core/asm/ResultHelpers.java
new file mode 100644
index 000000000..7b9b464b2
--- /dev/null
+++ b/src/main/java/dan200/computercraft/core/asm/ResultHelpers.java
@@ -0,0 +1,27 @@
+/*
+ * This file is part of ComputerCraft - http://www.computercraft.info
+ * Copyright Daniel Ratcliffe, 2011-2021. Do not distribute without permission.
+ * Send enquiries to dratcliffe@gmail.com
+ */
+package dan200.computercraft.core.asm;
+
+import dan200.computercraft.api.lua.MethodResult;
+
+final class ResultHelpers
+{
+ private ResultHelpers()
+ {
+ }
+
+ static Object[] checkNormalResult( MethodResult result )
+ {
+ if( result.getCallback() != null )
+ {
+ // Due to how tasks are implemented, we can't currently return a MethodResult. This is an
+ // entirely artificial limitation - we can remove it if it ever becomes an issue.
+ throw new IllegalStateException( "Must return MethodResult.of from mainThread function." );
+ }
+
+ return result.getResult();
+ }
+}
diff --git a/src/main/java/dan200/computercraft/core/computer/MainThread.java b/src/main/java/dan200/computercraft/core/computer/MainThread.java
index 8f53541e2..1c77e3692 100644
--- a/src/main/java/dan200/computercraft/core/computer/MainThread.java
+++ b/src/main/java/dan200/computercraft/core/computer/MainThread.java
@@ -93,7 +93,7 @@ public final class MainThread
executor.updateTime();
// We're not currently on the queue, so update its current execution time to
- // ensure its at least as high as the minimum.
+ // ensure it's at least as high as the minimum.
long newRuntime = minimumTime;
// Slow down new computers a little bit.
diff --git a/src/main/java/dan200/computercraft/core/filesystem/ResourceMount.java b/src/main/java/dan200/computercraft/core/filesystem/ResourceMount.java
index 2e6283066..d31757cda 100644
--- a/src/main/java/dan200/computercraft/core/filesystem/ResourceMount.java
+++ b/src/main/java/dan200/computercraft/core/filesystem/ResourceMount.java
@@ -7,7 +7,6 @@ package dan200.computercraft.core.filesystem;
import com.google.common.cache.Cache;
import com.google.common.cache.CacheBuilder;
-import com.google.common.collect.MapMaker;
import com.google.common.io.ByteStreams;
import dan200.computercraft.ComputerCraft;
import dan200.computercraft.api.filesystem.IMount;
@@ -15,10 +14,11 @@ import dan200.computercraft.core.apis.handles.ArrayByteChannel;
import dan200.computercraft.shared.util.IoUtil;
import net.minecraft.ResourceLocationException;
import net.minecraft.resources.ResourceLocation;
-import net.minecraft.server.packs.resources.ReloadableResourceManager;
+import net.minecraft.server.packs.resources.PreparableReloadListener;
import net.minecraft.server.packs.resources.Resource;
import net.minecraft.server.packs.resources.ResourceManager;
-import net.minecraft.server.packs.resources.ResourceManagerReloadListener;
+import net.minecraft.server.packs.resources.SimplePreparableReloadListener;
+import net.minecraft.util.profiling.ProfilerFiller;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;
@@ -27,7 +27,9 @@ import java.io.IOException;
import java.io.InputStream;
import java.nio.channels.Channels;
import java.nio.channels.ReadableByteChannel;
-import java.util.*;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
import java.util.concurrent.TimeUnit;
public final class ResourceMount implements IMount
@@ -56,50 +58,37 @@ public final class ResourceMount implements IMount
.weigher( ( k, v ) -> v.length )
.build();
- private static final MapMaker CACHE_TEMPLATE = new MapMaker().weakValues().concurrencyLevel( 1 );
-
/**
* Maintain a cache of currently loaded resource mounts. This cache is invalidated when currentManager changes.
*/
- private static final Map> MOUNT_CACHE = new WeakHashMap<>( 2 );
+ private static final Map MOUNT_CACHE = new HashMap<>( 2 );
private final String namespace;
private final String subPath;
- private final ReloadableResourceManager manager;
+ private ResourceManager manager;
@Nullable
private FileEntry root;
- public static ResourceMount get( String namespace, String subPath, ReloadableResourceManager manager )
+ public static ResourceMount get( String namespace, String subPath, ResourceManager manager )
{
- Map cache;
-
+ ResourceLocation path = new ResourceLocation( namespace, subPath );
synchronized( MOUNT_CACHE )
{
- cache = MOUNT_CACHE.get( manager );
- if( cache == null ) MOUNT_CACHE.put( manager, cache = CACHE_TEMPLATE.makeMap() );
- }
-
- ResourceLocation path = new ResourceLocation( namespace, subPath );
- synchronized( cache )
- {
- ResourceMount mount = cache.get( path );
- if( mount == null ) cache.put( path, mount = new ResourceMount( namespace, subPath, manager ) );
+ ResourceMount mount = MOUNT_CACHE.get( path );
+ if( mount == null ) MOUNT_CACHE.put( path, mount = new ResourceMount( namespace, subPath, manager ) );
return mount;
}
}
- private ResourceMount( String namespace, String subPath, ReloadableResourceManager manager )
+ private ResourceMount( String namespace, String subPath, ResourceManager manager )
{
this.namespace = namespace;
this.subPath = subPath;
- this.manager = manager;
-
- Listener.INSTANCE.add( manager, this );
- if( root == null ) load();
+ load( manager );
}
- private void load()
+ private void load( ResourceManager manager )
{
boolean hasAny = false;
String existingNamespace = null;
@@ -117,6 +106,7 @@ public final class ResourceMount implements IMount
hasAny = true;
}
+ this.manager = manager;
root = hasAny ? newRoot : null;
if( !hasAny )
@@ -292,28 +282,30 @@ public final class ResourceMount implements IMount
}
/**
- * A {@link ResourceManagerReloadListener} which reloads any associated mounts.
- *
- * While people should really be keeping a permanent reference to this, some people construct it every
- * method call, so let's make this as small as possible.
+ * A {@link PreparableReloadListener} which reloads any associated mounts and correctly updates the resource manager
+ * they point to.
*/
- static class Listener implements ResourceManagerReloadListener
+ public static final SimplePreparableReloadListener RELOAD_LISTENER = new SimplePreparableReloadListener<>()
{
- private static final Listener INSTANCE = new Listener();
-
- private final Set mounts = Collections.newSetFromMap( new WeakHashMap<>() );
- private final Set managers = Collections.newSetFromMap( new WeakHashMap<>() );
+ @Nonnull
+ @Override
+ protected Void prepare( @Nonnull ResourceManager manager, @Nonnull ProfilerFiller profiler )
+ {
+ profiler.push( "Reloading ComputerCraft mounts" );
+ try
+ {
+ for( ResourceMount mount : MOUNT_CACHE.values() ) mount.load( manager );
+ }
+ finally
+ {
+ profiler.pop();
+ }
+ return null;
+ }
@Override
- public void onResourceManagerReload( @Nonnull ResourceManager manager )
+ protected void apply( @Nonnull Void result, @Nonnull ResourceManager manager, @Nonnull ProfilerFiller profiler )
{
- for( ResourceMount mount : mounts ) mount.load();
}
-
- synchronized void add( ReloadableResourceManager manager, ResourceMount mount )
- {
- if( managers.add( manager ) ) manager.registerReloadListener( this );
- mounts.add( mount );
- }
- }
+ };
}
diff --git a/src/main/java/dan200/computercraft/core/lua/LuaContext.java b/src/main/java/dan200/computercraft/core/lua/LuaContext.java
index 0e5c792c8..626d15fad 100644
--- a/src/main/java/dan200/computercraft/core/lua/LuaContext.java
+++ b/src/main/java/dan200/computercraft/core/lua/LuaContext.java
@@ -9,8 +9,6 @@ import dan200.computercraft.ComputerCraft;
import dan200.computercraft.api.lua.ILuaContext;
import dan200.computercraft.api.lua.ILuaTask;
import dan200.computercraft.api.lua.LuaException;
-import dan200.computercraft.api.lua.MethodResult;
-import dan200.computercraft.core.asm.TaskCallback;
import dan200.computercraft.core.computer.Computer;
import dan200.computercraft.core.computer.MainThread;
@@ -68,11 +66,4 @@ class LuaContext implements ILuaContext
throw new LuaException( "Task limit exceeded" );
}
}
-
- @Nonnull
- @Override
- public MethodResult executeMainThreadTask( @Nonnull ILuaTask task ) throws LuaException
- {
- return TaskCallback.make( this, task );
- }
}
diff --git a/src/main/java/dan200/computercraft/shared/CommonHooks.java b/src/main/java/dan200/computercraft/shared/CommonHooks.java
index e3339cf99..9c2f36d2a 100644
--- a/src/main/java/dan200/computercraft/shared/CommonHooks.java
+++ b/src/main/java/dan200/computercraft/shared/CommonHooks.java
@@ -8,6 +8,7 @@ package dan200.computercraft.shared;
import dan200.computercraft.ComputerCraft;
import dan200.computercraft.core.apis.http.NetworkUtils;
import dan200.computercraft.core.computer.MainThread;
+import dan200.computercraft.core.filesystem.ResourceMount;
import dan200.computercraft.core.tracking.ComputerMBean;
import dan200.computercraft.core.tracking.Tracking;
import dan200.computercraft.shared.command.CommandComputerCraft;
@@ -23,6 +24,7 @@ import net.minecraft.world.level.storage.loot.BuiltInLootTables;
import net.minecraft.world.level.storage.loot.LootPool;
import net.minecraft.world.level.storage.loot.entries.LootTableReference;
import net.minecraft.world.level.storage.loot.providers.number.ConstantValue;
+import net.minecraftforge.event.AddReloadListenerEvent;
import net.minecraftforge.event.LootTableLoadEvent;
import net.minecraftforge.event.RegisterCommandsEvent;
import net.minecraftforge.event.TickEvent;
@@ -138,4 +140,10 @@ public final class CommonHooks
.name( "computercraft_treasure" )
.build() );
}
+
+ @SubscribeEvent
+ public static void onAddReloadListeners( AddReloadListenerEvent event )
+ {
+ event.addListener( ResourceMount.RELOAD_LISTENER );
+ }
}
diff --git a/src/main/java/dan200/computercraft/shared/common/TileGeneric.java b/src/main/java/dan200/computercraft/shared/common/TileGeneric.java
index cb74c0d25..a4bb2ca94 100644
--- a/src/main/java/dan200/computercraft/shared/common/TileGeneric.java
+++ b/src/main/java/dan200/computercraft/shared/common/TileGeneric.java
@@ -16,6 +16,7 @@ import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.world.phys.BlockHitResult;
+import net.minecraftforge.common.util.Constants;
import javax.annotation.Nonnull;
@@ -35,7 +36,7 @@ public abstract class TileGeneric extends BlockEntity
setChanged();
BlockPos pos = getBlockPos();
BlockState state = getBlockState();
- getLevel().sendBlockUpdated( pos, state, state, 3 );
+ getLevel().sendBlockUpdated( pos, state, state, Constants.BlockFlags.DEFAULT );
}
@Nonnull
diff --git a/src/main/java/dan200/computercraft/shared/computer/apis/CommandAPI.java b/src/main/java/dan200/computercraft/shared/computer/apis/CommandAPI.java
index 4b8e2b612..64a4c9bf4 100644
--- a/src/main/java/dan200/computercraft/shared/computer/apis/CommandAPI.java
+++ b/src/main/java/dan200/computercraft/shared/computer/apis/CommandAPI.java
@@ -15,12 +15,16 @@ import dan200.computercraft.shared.util.NBTUtil;
import net.minecraft.commands.CommandSourceStack;
import net.minecraft.commands.Commands;
import net.minecraft.core.BlockPos;
+import net.minecraft.core.Registry;
import net.minecraft.nbt.CompoundTag;
+import net.minecraft.resources.ResourceKey;
+import net.minecraft.resources.ResourceLocation;
import net.minecraft.server.MinecraftServer;
import net.minecraft.world.level.Level;
import net.minecraft.world.level.block.entity.BlockEntity;
import net.minecraft.world.level.block.state.BlockState;
+import javax.annotation.Nonnull;
import java.util.*;
/**
@@ -187,22 +191,24 @@ public class CommandAPI implements ILuaAPI
* Blocks are traversed by ascending y level, followed by z and x - the returned
* table may be indexed using `x + z*width + y*depth*depth`.
*
* @param minX The start x coordinate of the range to query.
* @param minY The start y coordinate of the range to query.
* @param minZ The start z coordinate of the range to query.
* @param maxX The end x coordinate of the range to query.
* @param maxY The end y coordinate of the range to query.
* @param maxZ The end z coordinate of the range to query.
* @return A list of information about each block.
* @throws LuaException If the coordinates are not within the world.
* @throws LuaException If trying to get information about more than 4096 blocks.
* @cc.since 1.76
+ * @cc.changed 1.99 Added {@code dimension} argument.
*/
@LuaFunction( mainThread = true )
- public final List
@@ -187,22 +191,24 @@ public class CommandAPI implements ILuaAPI
* table may be indexed using `x + z*width + y*depth*depth`.
*
- * @param minX The start x coordinate of the range to query.
- * @param minY The start y coordinate of the range to query.
- * @param minZ The start z coordinate of the range to query.
- * @param maxX The end x coordinate of the range to query.
- * @param maxY The end y coordinate of the range to query.
- * @param maxZ The end z coordinate of the range to query.
+ * @param minX The start x coordinate of the range to query.
+ * @param minY The start y coordinate of the range to query.
+ * @param minZ The start z coordinate of the range to query.
+ * @param maxX The end x coordinate of the range to query.
+ * @param maxY The end y coordinate of the range to query.
+ * @param maxZ The end z coordinate of the range to query.
+ * @param dimension The dimension to query (e.g. "minecraft:overworld"). Defaults to the current dimension.
* @return A list of information about each block.
* @throws LuaException If the coordinates are not within the world.